@@ -10,7 +10,7 @@ from matplotlib import pyplot
 from . import heatmapping
 from . import lines
 from . import plotting
-from .helpers import project_point
+from .helpers import project_point, convert_coordinates_sequence
 
 
 def figure(ax=None, scale=None, permutation=None):
@@ -29,20 +29,22 @@ def figure(ax=None, scale=None, permutation=None):
     ternary_ax = TernaryAxesSubplot(ax=ax, scale=scale, permutation=permutation)
     return ternary_ax.get_figure(), ternary_ax
 
+
 def mpl_redraw_callback(event, tax):
     """
-    Callback to properly rotate and redraw text labels when the plot is drawn 
+    Callback to properly rotate and redraw text labels when the plot is drawn
     or resized.
 
-    Parameters:
+    Parameters
+    ----------
     event: a matplotlib event
         either 'resize_event' or 'draw_event'
     tax: TernaryAxesSubplot
-         the TernaryAxesSubplot 
+         the TernaryAxesSubplot
     """
-
     tax._redraw_labels()
 
+
 class TernaryAxesSubplot(object):
     """
     Wrapper for python-ternary and matplotlib figure. Parameters for member
@@ -61,19 +63,19 @@ class TernaryAxesSubplot(object):
         self.set_scale(scale=scale)
         self._permutation = permutation
         self._boundary_scale = scale
-        self._labels = dict() # Container for the axis labels supplied by the user
+        # Container for the axis labels supplied by the user
+        self._labels = dict()
+        self._corner_labels = dict()
         self._ticks = dict()
-        self._to_remove = [] # Container for the redrawing of labels
+        # Container for the redrawing of labels
+        self._to_remove = []
         self._connect_callbacks()
 
     def _connect_callbacks(self):
-        """
-        Connect resize matplotlib callbacks.
-        """
-
+        """Connect resize matplotlib callbacks."""
         figure = self.get_figure()
         callback = partial(mpl_redraw_callback, tax=self)
-        event_names = ('resize_event', )
+        event_names = ('resize_event', 'draw_event')
         for event_name in event_names:
             figure.canvas.mpl_connect(event_name, callback)
 
@@ -81,17 +83,11 @@ class TernaryAxesSubplot(object):
         return "TernaryAxesSubplot: %s" % self.ax.__hash__()
 
     def get_axes(self):
-        """
-        Return the underlying matplotlib AxesSubplot object
-        """
-
+        """Returns the underlying matplotlib AxesSubplot object."""
         return self.ax
 
     def get_figure(self):
-        """
-        Return the underlying matplotlib figure object.
-        """
-
+        """Return the underlying matplotlib figure object."""
         ax = self.get_axes()
         return ax.get_figure()
 
@@ -102,13 +98,24 @@ class TernaryAxesSubplot(object):
     def get_scale(self):
         return self._scale
 
+    def set_axis_limits(self, axis_limits=None):
+        """
+        Set min and max data limits for each of the three axes.
+
+        axis_limits = dict
+            keys are 'b','l' and 'r' for the three axes
+            vals are lists of the min and max values for the axis in
+            data units.
+        """
+        self._axis_limits = axis_limits
+
+    def get_axis_limits(self):
+        return self._axis_limits
+
     # Title and Axis Labels
 
     def set_title(self, title, **kwargs):
-        """
-        Sets the title on the underlying matplotlib AxesSubplot
-        """
-
+        """Sets the title on the underlying matplotlib AxesSubplot."""
         ax = self.get_axes()
         ax.set_title(title, **kwargs)
 
@@ -119,8 +126,6 @@ class TernaryAxesSubplot(object):
 
         Parameters
         ----------
-        ax: Matplotlib AxesSubplot, None
-            The subplot to draw on.
         label: String
             The axis label
         position: 3-Tuple of floats, None
@@ -139,13 +144,12 @@ class TernaryAxesSubplot(object):
 
     def right_axis_label(self, label, position=None, rotation=-60, offset=0.08,
                          **kwargs):
+
         """
         Sets the label on the right axis.
 
         Parameters
         ----------
-        ax: Matplotlib AxesSubplot, None
-            The subplot to draw on.
         label: String
             The axis label
         position: 3-Tuple of floats, None
@@ -159,7 +163,7 @@ class TernaryAxesSubplot(object):
         """
 
         if not position:
-            position = (2./5 + offset, 3./5, 0)
+            position = (2. / 5 + offset, 3. / 5, 0)
         self._labels["right"] = (label, position, rotation, kwargs)
 
     def bottom_axis_label(self, label, position=None, rotation=0, offset=0.02,
@@ -169,8 +173,6 @@ class TernaryAxesSubplot(object):
 
         Parameters
         ----------
-        ax: Matplotlib AxesSubplot, None
-            The subplot to draw on.
         label: String
             The axis label
         position: 3-Tuple of floats, None
@@ -184,8 +186,77 @@ class TernaryAxesSubplot(object):
         """
 
         if not position:
-            position = (1./2, offset, 1./2)
-        self._labels["bottom"] = (label, position, rotation, kwargs)
+            position = (0.5, -offset / 2., 0.5)
+        self._corner_labels["bottom"] = (label, position, rotation, kwargs)
+
+    def right_corner_label(self, label, position=None, rotation=0, offset=0.08,
+                           **kwargs):
+        """
+        Sets the label on the right corner (complements left axis).
+
+        Parameters
+        ----------
+        label: String
+            The axis label
+        position: 3-Tuple of floats, None
+            The position of the text label
+        rotation: float, 0
+            The angle of rotation of the label
+        offset: float,
+            Used to compute the distance of the label from the axis
+        kwargs:
+            Any kwargs to pass through to matplotlib.
+        """
+
+        if not position:
+            position = (1, offset / 2, 0)
+        self._labels["right"] = (label, position, rotation, kwargs)
+
+    def left_corner_label(self, label, position=None, rotation=0, offset=0.08,
+                          **kwargs):
+        """
+        Sets the label on the left corner (complements right axis.)
+
+        Parameters
+        ----------
+        label: string
+            The axis label
+        position: 3-Tuple of floats, None
+            The position of the text label
+        rotation: float, 0
+            The angle of rotation of the label
+        offset: float,
+            Used to compute the distance of the label from the axis
+        kwargs:
+            Any kwargs to pass through to matplotlib.
+        """
+
+        if not position:
+            position = (-offset / 2, offset / 2, 0)
+        self._corner_labels["left"] = (label, position, rotation, kwargs)
+
+    def top_corner_label(self, label, position=None, rotation=0, offset=0.2,
+                         **kwargs):
+        """
+        Sets the label on the bottom axis.
+
+        Parameters
+        ----------
+        label: String
+            The axis label
+        position: 3-Tuple of floats, None
+            The position of the text label
+        rotation: float, 0
+            The angle of rotation of the label
+        offset: float,
+            Used to compute the distance of the label from the axis
+        kwargs:
+            Any kwargs to pass through to matplotlib.
+        """
+
+        if not position:
+            position = (-offset / 2, 1 + offset, 0)
+        self._corner_labels["bottom"] = (label, position, rotation, kwargs)
 
     def annotate(self, text, position, **kwargs):
         ax = self.get_axes()
@@ -197,7 +268,7 @@ class TernaryAxesSubplot(object):
     def boundary(self, scale=None, axes_colors=None, **kwargs):
         # Sometimes you want to draw a bigger boundary
         if not scale:
-            scale = self._boundary_scale # defaults to self._scale
+            scale = self._boundary_scale  # defaults to self._scale
         ax = self.get_axes()
         self.resize_drawing_canvas(scale)
         lines.boundary(scale=scale, ax=ax, axes_colors=axes_colors, **kwargs)
@@ -220,7 +291,7 @@ class TernaryAxesSubplot(object):
     def horizontal_line(self, i, **kwargs):
         ax = self.get_axes()
         scale = self.get_scale()
-        lines.horizontal_line(ax, scale, i, **kwargs) 
+        lines.horizontal_line(ax, scale, i, **kwargs)
 
     def left_parallel_line(self, i, **kwargs):
         ax = self.get_axes()
@@ -234,34 +305,63 @@ class TernaryAxesSubplot(object):
 
     # Matplotlib passthroughs
 
+    def close(self):
+        fig = self.get_figure()
+        pyplot.close(fig)
+
     def legend(self, *args, **kwargs):
         ax = self.get_axes()
         ax.legend(*args, **kwargs)
 
     def savefig(self, filename, dpi=200, format=None):
-        figure = self.get_figure()
-        figure.savefig(filename, format=format, dpi=dpi)
+        self._redraw_labels()
+        fig = self.get_figure()
+        fig.savefig(filename, format=format, dpi=dpi)
 
     def show(self):
+        self._redraw_labels()
         pyplot.show()
 
     # Axis ticks
 
     def clear_matplotlib_ticks(self, axis="both"):
-        """
-        Clears the default matplotlib ticks.
-        """
-
+        """Clears the default matplotlib ticks."""
         ax = self.get_axes()
         plotting.clear_matplotlib_ticks(ax=ax, axis=axis)
 
+    def get_ticks_from_axis_limits(self, multiple=1):
+        """
+        Taking self._axis_limits and self._boundary_scale get the scaled
+        ticks for all three axes and store them in self._ticks under the
+        keys 'b' for bottom, 'l' for left and 'r' for right axes.
+        """
+        for k in ['b', 'l', 'r']:
+            self._ticks[k] = numpy.linspace(
+                self._axis_limits[k][0],
+                self._axis_limits[k][1],
+                self._boundary_scale / float(multiple) + 1
+            ).tolist()
+
+    def set_custom_ticks(self, locations=None, clockwise=False, multiple=1,
+                         axes_colors=None, tick_formats=None, **kwargs):
+        """
+        Having called get_ticks_from_axis_limits, set the custom ticks on the
+        plot.
+        """
+        for k in ['b', 'l', 'r']:
+            self.ticks(ticks=self._ticks[k], locations=locations,
+                       axis=k, clockwise=clockwise, multiple=multiple,
+                       axes_colors=axes_colors, tick_formats=tick_formats,
+                       **kwargs)
+
     def ticks(self, ticks=None, locations=None, multiple=1, axis='blr',
-              clockwise=False, axes_colors=None, **kwargs):
+              clockwise=False, axes_colors=None, tick_formats=None, **kwargs):
         ax = self.get_axes()
         scale = self.get_scale()
         lines.ticks(ax, scale, ticks=ticks, locations=locations,
-                    multiple=multiple, clockwise=clockwise, axis=axis, 
-                    axes_colors=axes_colors, **kwargs)
+                    multiple=multiple, clockwise=clockwise, axis=axis,
+                    axes_colors=axes_colors, tick_formats=tick_formats,
+                    **kwargs)
 
     # Redrawing and resizing
 
@@ -272,28 +372,37 @@ class TernaryAxesSubplot(object):
         plotting.resize_drawing_canvas(ax, scale=scale)
 
     def _redraw_labels(self):
-        """
-        Redraw axis labels, typically after draw or resize events.
-        """
-
+        """Redraw axis labels, typically after draw or resize events."""
         ax = self.get_axes()
         # Remove any previous labels
         for mpl_object in self._to_remove:
             mpl_object.remove()
         self._to_remove = []
         # Redraw the labels with the appropriate angles
-        for (label, position, rotation, kwargs) in self._labels.values():
+        label_data = list(self._labels.values())
+        label_data.extend(self._corner_labels.values())
+        for (label, position, rotation, kwargs) in label_data:
             transform = ax.transAxes
             x, y = project_point(position)
             # Calculate the new angle.
-            position = numpy.array([x,y])
-            new_rotation = ax.transData.transform_angles(numpy.array((rotation,)), position.reshape((1,2)))[0]
-            text = ax.text(x, y, label, rotation=new_rotation, transform=transform,
-                        horizontalalignment="center", **kwargs)
+            position = numpy.array([x, y])
+            new_rotation = ax.transData.transform_angles(
+                numpy.array((rotation,)), position.reshape((1, 2)))[0]
+            text = ax.text(x, y, label, rotation=new_rotation,
+                           transform=transform, horizontalalignment="center",
+                           **kwargs)
             text.set_rotation_mode("anchor")
             self._to_remove.append(text)
 
-    ## Various Plots
+    def convert_coordinates(self, points, axisorder='blr'):
+        """
+        Convert data coordinates to simplex coordinates for plotting
+        in the case that axis limits have been applied.
+        """
+        return convert_coordinates_sequence(points,self._boundary_scale,
+                                            self._axis_limits, axisorder)
+
+    # Various Plots
 
     def scatter(self, points, **kwargs):
         ax = self.get_axes()
@@ -316,7 +425,7 @@ class TernaryAxesSubplot(object):
 
     def heatmap(self, data, scale=None, cmap=None, scientific=False,
                 style='triangular', colorbar=True, colormap=True,
-                vmin=None, vmax=None, cbarlabel=None):
+                vmin=None, vmax=None, cbarlabel=None, cb_kwargs=None):
         permutation = self._permutation
         if not scale:
             scale = self.get_scale()
@@ -326,11 +435,12 @@ class TernaryAxesSubplot(object):
         heatmapping.heatmap(data, scale, cmap=cmap, style=style, ax=ax,
                             scientific=scientific, colorbar=colorbar,
                             permutation=permutation, colormap=colormap,
-                            vmin=vmin, vmax=vmax, cbarlabel=cbarlabel)
+                            vmin=vmin, vmax=vmax, cbarlabel=cbarlabel,
+                            cb_kwargs=cb_kwargs)
 
     def heatmapf(self, func, scale=None, cmap=None, boundary=True,
                  style='triangular', colorbar=True, scientific=True,
-                 vmin=None, vmax=None, cbarlabel=None):
+                 vmin=None, vmax=None, cbarlabel=None, cb_kwargs=None):
         if not scale:
             scale = self.get_scale()
         if style.lower()[0] == 'd':
@@ -340,4 +450,5 @@ class TernaryAxesSubplot(object):
         heatmapping.heatmapf(func, scale, cmap=cmap, style=style,
                              boundary=boundary, ax=ax, scientific=scientific,
                              colorbar=colorbar, permutation=permutation,
-                             vmin=vmin, vmax=vmax, cbarlabel=cbarlabel)
+                             vmin=vmin, vmax=vmax, cbarlabel=cbarlabel,
+                             cb_kwargs=cb_kwargs)
